"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const http = require("http");
const querystring = require("query-string");
const request = require("request-promise");
class CouchDb {
    constructor(options = {}) {
        this.host = options.host || 'http://127.0.0.1';
        this.port = options.port || 5984;
        this.logging = options.logging || false;
        this.defaultDatabase = options.defaultDatabase;
        if (options.auth) {
            try {
                this.auth = JSON.parse(JSON.stringify(options.auth));
            }
            catch (_a) {
                console.error('Error configuring Node CouchDB Client auth');
            }
        }
    }
    createQueryString(queryObj = {}) {
        return Object.keys(queryObj).length ? `?${querystring.stringify(queryObj)}` : '';
    }
    get defaultRequestOptions() {
        return {
            uri: `${this.host}:${this.port}`,
            auth: this.auth,
            method: 'GET',
            json: true,
            headers: {
                'user-agent': 'couchdb-promises',
                'accept': 'application/json'
            },
            resolveWithFullResponse: true
        };
    }
    request(options = {}) {
        const startTime = Date.now();
        const requestOptions = this.defaultRequestOptions;
        requestOptions.method = options.method || requestOptions.method;
        requestOptions.uri += `/${options.path}`;
        requestOptions.body = options.postData;
        if (options.headers && options.headers.length > 0) {
            requestOptions.headers = Object.assign({}, requestOptions.headers, options.headers);
        }
        return request(requestOptions).then(response => {
            if (this.logging) {
                console.info({
                    statusCode: response.statusCode,
                    message: this.statusCode(options.statusCodes, response.statusCode),
                    body: response.body,
                    duration: Date.now() - startTime
                });
            }
            return response.body;
        });
    }
    statusCode(statusCodes, status) {
        const codes = Object.assign({}, http.STATUS_CODES, statusCodes);
        return codes[status] || 'unknown status';
    }
    getInfo() {
        return this.request({
            statusCodes: {
                200: 'OK - Request completed successfully'
            }
        });
    }
    getDatabases() {
        return this.request({
            path: '_all_dbs',
            statusCodes: {
                200: 'OK - Request completed successfully',
                401: 'Unauthorized - CouchDB Server Administrator privileges required',
            }
        });
    }
    getDatabase(dbName) {
        dbName = dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        return this.request({
            path: encodeURIComponent(dbName),
            statusCodes: {
                200: 'OK - Request completed successfully',
                401: 'Unauthorized - CouchDB Server Administrator privileges required',
                404: 'Not Found – Requested database not found'
            }
        });
    }
    getDatabaseRevisionLimit(dbName) {
        return this.request({
            path: `${encodeURIComponent(dbName)}/${encodeURIComponent('_revs_limit')}`,
            method: 'GET',
            statusCodes: {
                200: 'OK - Request completed successfully'
            }
        });
    }
    checkDatabaseExists(dbName) {
        dbName = dbName || this.defaultDatabase;
        if (!dbName) {
            throw new Error('No DB specified. Set defaultDatabase or specify one');
        }
        return this.request({
            path: encodeURIComponent(dbName),
            method: 'HEAD',
            statusCodes: {
                200: 'OK - Database exists',
                401: 'Unauthorized - CouchDB Server Administrator privileges required',
                404: 'Not Found – Requested database not found'
            }
        }).then(response => {
            return true;
        }, error => {
            if (error && error.statusCode === 404) {
                return false;
            }
            throw error;
        });
    }
    createDatabase(dbName, options) {
        return __awaiter(this, void 0, void 0, function* () {
            dbName = dbName || this.defaultDatabase;
            if (!dbName) {
                return Promise.reject('No DB specified. Set defaultDatabase or specify one');
            }
            if (dbName !== '_users' && !dbName.match('^[a-z][a-z0-9_$()+/-]*$')) {
                return Promise.reject(`Invalid DB Name '${dbName}': http://docs.couchdb.org/en/latest/api/database/common.html#put--db`);
            }
            return this.request({
                path: encodeURIComponent(dbName),
                method: 'PUT',
                statusCodes: {
                    201: 'Created - Database created successfully',
                    400: 'Bad Request - Invalid database name',
                    401: 'Unauthorized - CouchDB Server Administrator privileges required',
                    412: 'Precondition Failed - Database already exists'
                }
            }).then((response) => __awaiter(this, void 0, void 0, function* () {
                if (options && options.revisionLimit) {
                    yield this.setDatabaseRevisionLimit(dbName, options.revisionLimit);
                }
                return response;
            }), error => {
                throw error;
            });
        });
    }
    compactDatabase(dbName) {
        return this.request({
            path: `${encodeURIComponent(dbName)}/_compact`,
            method: 'POST',
            postData: {},
            statusCodes: {
                202: 'Accepted - Compaction request has been accepted',
                400: 'Bad Request - Invalid database name',
                401: 'Unauthorized - CouchDB Server Administrator privileges required',
                415: 'Unsupported Media Type - Bad Content-Type value'
            }
        });
    }
    setDatabaseRevisionLimit(dbName, revisionLimit) {
        return this.request({
            path: `${encodeURIComponent(dbName)}/_revs_limit`,
            method: 'PUT',
            postData: revisionLimit,
            statusCodes: {
                200: 'OK - Request completed successfully',
                400: 'Bad Request - Invalid JSON data'
            }
        });
    }
    deleteDatabase(dbName) {
        dbName = dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        return this.request({
            path: encodeURIComponent(dbName),
            method: 'DELETE',
            statusCodes: {
                200: 'OK - Database removed successfully',
                400: 'Bad Request - Invalid database name or forgotten document id by accident',
                401: 'Unauthorized - CouchDB Server Administrator privileges required',
                404: 'Not Found - Database doesn’t exist'
            }
        });
    }
    updateDatabaseSecurity(options) {
        const dbName = options.dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        return this.request({
            path: `${encodeURIComponent(dbName)}/_security`,
            method: 'PUT',
            postData: {
                admins: options.admins,
                members: options.members
            },
            statusCodes: {
                200: 'OK - Security updated',
                401: 'Unauthorized - CouchDB Server Administrator privileges required'
            }
        });
    }
    createUser(options) {
        options.roles = options.roles || [];
        return this.request({
            path: `_users`,
            method: 'POST',
            postData: {
                _id: `org.couchdb.user:${options.username}`,
                name: options.username,
                type: 'user',
                roles: [...options.roles],
                password: options.password
            },
            statusCodes: {
                201: 'Created New User',
                400: 'Bad Request - Invalid request',
                401: 'Unauthorized - CouchDB Server Administrator privileges required',
                500: 'Internal Server Error - Query execution error'
            }
        });
    }
    checkUserExists(username) {
        const dbName = '_users';
        return this.request({
            path: `${encodeURIComponent(dbName)}/org.couchdb.user:${username}`,
            method: 'HEAD',
            statusCodes: {
                200: 'OK - User exists',
                401: 'Unauthorized - Read privilege required',
                404: 'Not Found - User not found'
            }
        }).then(response => {
            return true;
        }, error => {
            if (error && error.statusCode === 404) {
                return false;
            }
            throw error;
        });
    }
    getUser(username) {
        const dbName = '_users';
        const docId = `org.couchdb.user:${username}`;
        return this.request({
            path: `${encodeURIComponent(dbName)}/${encodeURIComponent(docId)}`,
            statusCodes: {
                200: 'OK - Request completed successfully',
                304: 'Not Modified - Document wasn’t modified since specified revision',
                400: 'Bad Request - The format of the request or revision was invalid',
                401: 'Unauthorized - Read privilege required',
                404: 'Not Found - Document not found'
            }
        });
    }
    updateUserPassword(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const user = yield this.getUser(options.username);
            const dbName = '_users';
            return this.request({
                path: `${encodeURIComponent(dbName)}/${user._id}`,
                method: 'PUT',
                postData: {
                    _rev: user._rev,
                    _id: user._id,
                    name: user.name,
                    type: 'user',
                    roles: [...user.roles],
                    password: options.password
                },
                statusCodes: {
                    200: 'OK - User exists',
                    401: 'Unauthorized - Read privilege required',
                    404: 'Not Found - User not found',
                    409: 'Document update conflict',
                    500: 'Internal Server Error - Query execution error'
                }
            });
        });
    }
    findDocuments(options) {
        const dbName = options.dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        return this.request({
            path: `${encodeURIComponent(dbName)}/_find`,
            method: 'POST',
            postData: options.findOptions,
            statusCodes: {
                200: 'OK - Request completed successfully',
                400: 'Bad Request - Invalid request',
                401: 'Unauthorized - Read permission required',
                500: 'Internal Server Error - Query execution error'
            }
        });
    }
    getDocuments(options) {
        const dbName = options.dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        const queryStr = this.createQueryString(options.options);
        return this.request({
            path: `${encodeURIComponent(dbName)}/_all_docs${queryStr}`,
            statusCodes: {
                200: 'OK - Request completed successfully',
                401: 'Unauthorized - Read privilege required',
            }
        });
    }
    getDocument(options) {
        const dbName = options.dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        return this.request({
            path: `${encodeURIComponent(dbName)}/${encodeURIComponent(options.docId)}`,
            statusCodes: {
                200: 'OK - Request completed successfully',
                304: 'Not Modified - Document wasn’t modified since specified revision',
                400: 'Bad Request - The format of the request or revision was invalid',
                401: 'Unauthorized - Read privilege required',
                404: 'Not Found - Document not found'
            }
        });
    }
    checkDocumentExists(options) {
        const dbName = options.dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        return this.request({
            path: `${encodeURIComponent(dbName)}/${encodeURIComponent(options.docId)}`,
            method: 'HEAD',
            statusCodes: {
                200: 'OK - Document exists',
                304: 'Not Modified - Document wasn’t modified since specified revision',
                401: 'Unauthorized - Read privilege required',
                404: 'Not Found - Document not found'
            }
        }).then(response => {
            return true;
        }, error => {
            if (error && error.statusCode === 404) {
                return false;
            }
            throw error;
        });
    }
    copyDocument(options) {
        const dbName = options.dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        if (options.docId && options.newDocId) {
            return this.request({
                path: `${encodeURIComponent(dbName)}/${encodeURIComponent(options.docId)}`,
                method: 'COPY',
                headers: { Destination: options.newDocId },
                statusCodes: {
                    201: 'Created – Document created and stored on disk',
                    202: 'Accepted – Document data accepted, but not yet stored on disk',
                    400: 'Bad Request – Invalid request body or parameters',
                    401: 'Unauthorized – Write privileges required',
                    404: 'Not Found – Specified database or document ID doesn’t exists',
                    409: 'Conflict – Document with the specified ID already exists or specified revision is not latest for target document'
                }
            });
        }
        return Promise.reject('Invalid docId or newDocId');
    }
    createDocument(options) {
        const dbName = options.dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        if (options.docId) {
            return this.request({
                path: `${encodeURIComponent(dbName)}/${encodeURIComponent(options.docId)}`,
                method: 'PUT',
                postData: options.doc,
                headers: {
                    'content-type': 'application/json'
                },
                statusCodes: {
                    201: 'Created – Document created and stored on disk',
                    202: 'Accepted – Document data accepted, but not yet stored on disk',
                    400: 'Bad Request – Invalid request body or parameters',
                    401: 'Unauthorized – Write privileges required',
                    404: 'Not Found – Specified database or document ID doesn’t exists',
                    409: 'Conflict – Document with the specified ID already exists or specified revision is not latest for target document'
                }
            });
        }
        return this.request({
            path: encodeURIComponent(dbName),
            method: 'POST',
            postData: options.doc,
            statusCodes: {
                201: 'Created – Document created and stored on disk',
                202: 'Accepted – Document data accepted, but not yet stored on disk',
                400: 'Bad Request – Invalid database name',
                401: 'Unauthorized – Write privileges required',
                404: 'Not Found – Database doesn’t exists',
                409: 'Conflict – A Conflicting Document with same ID already exists'
            }
        });
    }
    upsertDocuments(options) {
        const dbName = options.dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        return this.request({
            path: `${encodeURIComponent(dbName)}/_bulk_docs`,
            method: 'POST',
            postData: {
                docs: options.docs
            },
            statusCodes: {
                201: 'Created – Document(s) have been created or updated',
                400: 'Bad Request – The request provided invalid JSON data',
                417: 'Expectation Failed – Occurs when all_or_nothing option set as true and at least one document was rejected by validation function',
                500: 'Internal Server Error – Malformed data provided, while it’s still valid JSON'
            }
        });
    }
    updateDocument(options) {
        const dbName = options.dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        return this.request({
            path: `${encodeURIComponent(dbName)}/${encodeURIComponent(options.docId)}`,
            method: 'PUT',
            headers: {
                'If-Match': options.rev
            },
            postData: options.updatedDoc,
            statusCodes: {
                201: 'Updated – Document(s) have been updated',
                400: 'Bad Request – The request provided invalid JSON data',
                409: 'Document Conflict',
                417: 'Expectation Failed – Occurs when all_or_nothing option set as true and at least one document was rejected by validation function',
                500: 'Internal Server Error – Malformed data provided, while it’s still valid JSON'
            }
        });
    }
    deleteDocument(options) {
        const dbName = options.dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        return this.request({
            path: `${encodeURIComponent(dbName)}/${encodeURIComponent(options.docId)}?rev=${options.rev}`,
            method: 'DELETE',
            statusCodes: {
                200: 'OK - Document successfully removed',
                202: 'Accepted - Request was accepted, but changes are not yet stored on disk',
                400: 'Bad Request - Invalid request body or parameters',
                401: 'Unauthorized - Write privilege required',
                404: 'Not Found - Specified database or document ID doesn\'t exist',
                409: 'Conflict - Specified revision is not the latest for target document'
            }
        });
    }
    getUuids(count = 1) {
        return this.request({
            path: `_uuids?count=${count}`,
            statusCodes: {
                200: 'OK - Request completed successfully',
                403: 'Forbidden – Requested more UUIDs than is allowed to retrieve'
            }
        });
    }
    getDesignDocument(options) {
        const dbName = options.dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        const queryStr = this.createQueryString(options.options);
        return this.request({
            path: `${encodeURIComponent(dbName)}/_design/${encodeURIComponent(options.docId)}${queryStr}`,
            statusCodes: {
                200: 'OK - Request completed successfully',
                304: 'Not Modified - Document wasn’t modified since specified revision',
                400: 'Bad Request - The format of the request or revision was invalid',
                401: 'Unauthorized - Read privilege required',
                404: 'Not Found - Document not found'
            }
        });
    }
    getDesignDocumentInfo(options) {
        const dbName = options.dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        return this.request({
            path: `${encodeURIComponent(dbName)}/_design/${encodeURIComponent(options.docId)}/_info`,
            statusCodes: {
                200: 'OK - Request completed successfully'
            }
        });
    }
    createDesignDocument(options) {
        const dbName = options.dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        return this.request({
            path: `${encodeURIComponent(dbName)}/_design/${encodeURIComponent(options.docId)}`,
            method: 'PUT',
            postData: options.doc,
            statusCodes: {
                201: 'Created – Document created and stored on disk',
                202: 'Accepted – Document data accepted, but not yet stored on disk',
                400: 'Bad Request – Invalid request body or parameters',
                401: 'Unauthorized – Write privileges required',
                404: 'Not Found – Specified database or document ID doesn’t exists',
                409: 'Conflict – Document with the specified ID already exists or specified revision is not latest for target document'
            }
        });
    }
    deleteDesignDocument(options) {
        const dbName = options.dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        return this.request({
            path: `${encodeURIComponent(dbName)}/_design/${encodeURIComponent(options.docId)}?rev=${options.rev}`,
            method: 'DELETE',
            statusCodes: {
                200: 'OK - Document successfully removed',
                202: 'Accepted - Request was accepted, but changes are not yet stored on disk',
                400: 'Bad Request - Invalid request body or parameters',
                401: 'Unauthorized - Write privilege required',
                404: 'Not Found - Specified database or document ID doesn\'t exist',
                409: 'Conflict - Specified revision is not the latest for target document'
            }
        });
    }
    getDocumentView(options) {
        const dbName = options.dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        const queryStr = this.createQueryString(options.options);
        return this.request({
            path: `${encodeURIComponent(dbName)}/_design/${encodeURIComponent(options.docId)}/_view/${encodeURIComponent(options.viewName)}${queryStr}`,
            statusCodes: {
                200: 'OK - Request completed successfully'
            }
        });
    }
    getIndexes(dbName) {
        dbName = dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        return this.request({
            path: `${encodeURIComponent(dbName)}/_index`,
            method: 'GET',
            statusCodes: {
                200: 'OK - Success',
                400: 'Bad Request - Invalid request',
                401: 'Unauthorized - Read permission required',
                500: 'Internal Server Error - Execution error'
            }
        });
    }
    createIndex(options) {
        const dbName = options.dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        return this.request({
            path: `${encodeURIComponent(dbName)}/_index`,
            method: 'POST',
            postData: {
                index: options.index,
                name: options.name
            },
            statusCodes: {
                200: 'OK - Index created successfully or already exists',
                400: 'Bad Request - Invalid request',
                401: 'Unauthorized - Admin permission required',
                500: 'Internal Server Error - Execution error'
            }
        });
    }
    deleteIndex(options) {
        const dbName = options.dbName || this.defaultDatabase;
        if (!dbName) {
            return Promise.reject('No DB specified. Set defaultDatabase or specify one');
        }
        return this.request({
            path: `${encodeURIComponent(dbName)}/_index/${encodeURIComponent(options.docId)}/json/${encodeURIComponent(options.indexName)}`,
            method: 'DELETE',
            statusCodes: {
                200: 'OK - Success',
                400: 'Bad Request - Invalid request',
                401: 'Unauthorized - Writer permission required',
                404: 'Not Found - Index not found',
                500: 'Internal Server Error - Execution error'
            }
        });
    }
}
exports.CouchDb = CouchDb;
